<?php

namespace B2buy\Coupa\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use B2buy\Coupa\Http\Requests\CoupaRequest;
use B2buy\Coupa\Interfaces\CoupaRepositoryInterface;
use B2buy\Coupa\Models\Coupa;
use B2buy\Coupa\Forms\Coupa as Form;

/**
 * APIController  class for coupa.
 */
class CoupaAPIController extends BaseController
{

    /**
     * Initialize coupa resource controller.
     *
     * @param type CoupaRepositoryInterface $coupa
     *
     * @return null
     */
    public function __construct(CoupaRepositoryInterface $coupa)
    {
        parent::__construct();
        $this->repository = $coupa;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\B2buy\Coupa\Repositories\Criteria\CoupaResourceCriteria::class);
    }

    /**
     * Display a list of coupa.
     *
     * @return Response
     */
    public function index(CoupaRequest $request)
    {
        return $this->repository
            ->setPresenter(\B2buy\Coupa\Repositories\Presenter\CoupaPresenter::class)
            ->paginate();
    }

    /**
     * Display coupa.
     *
     * @param Request $request
     * @param Model   $coupa
     *
     * @return Response
     */
    public function show(CoupaRequest $request, Coupa $coupa)
    {
        return $coupa->setPresenter(\B2buy\Coupa\Repositories\Presenter\CoupaListPresenter::class);
        ;
    }

    /**
     * Create new coupa.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CoupaRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('coupa::coupa.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('coupa/coupa/' . $coupa->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('coupa/coupa');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the coupa.
     *
     * @param Request $request
     * @param Model   $coupa
     *
     * @return Response
     */
    public function update(CoupaRequest $request, Coupa $coupa)
    {
        try {
            $data = $request->all();

            $coupa->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('coupa::coupa.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('coupa/coupa/' . $coupa->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('coupa/coupa/' . $coupa->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the coupa.
     *
     * @param Model   $coupa
     *
     * @return Response
     */
    public function destroy(CoupaRequest $request, Coupa $coupa)
    {
        try {
            $coupa->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('coupa::coupa.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('coupa/coupa/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('coupa/coupa/' . $coupa->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
