<?php

namespace B2buy\Coupa\Policies;

use Litepie\User\Contracts\UserPolicy;
use B2buy\Coupa\Models\Coupa;

class CoupaPolicy
{

    /**
     * Determine if the given user can view the coupa.
     *
     * @param UserPolicy $user
     * @param Coupa $coupa
     *
     * @return bool
     */
    public function view(UserPolicy $user, Coupa $coupa)
    {
        if ($user->canDo('coupa.coupa.view') && $user->isAdmin()) {
            return true;
        }

        return $coupa->user_id == user_id() && $coupa->user_type == user_type();
    }

    /**
     * Determine if the given user can create a coupa.
     *
     * @param UserPolicy $user
     * @param Coupa $coupa
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('coupa.coupa.create');
    }

    /**
     * Determine if the given user can update the given coupa.
     *
     * @param UserPolicy $user
     * @param Coupa $coupa
     *
     * @return bool
     */
    public function update(UserPolicy $user, Coupa $coupa)
    {
        if ($user->canDo('coupa.coupa.edit') && $user->isAdmin()) {
            return true;
        }

        return $coupa->user_id == user_id() && $coupa->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given coupa.
     *
     * @param UserPolicy $user
     * @param Coupa $coupa
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Coupa $coupa)
    {
        return $coupa->user_id == user_id() && $coupa->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given coupa.
     *
     * @param UserPolicy $user
     * @param Coupa $coupa
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Coupa $coupa)
    {
        if ($user->canDo('coupa.coupa.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given coupa.
     *
     * @param UserPolicy $user
     * @param Coupa $coupa
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Coupa $coupa)
    {
        if ($user->canDo('coupa.coupa.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
